((Drupal, settings, once) => {
  /**
   * Dispatches a custom 'altcolor-changed' event with the new color array in
   * the event details. This allows other JavaScripts to listen to updating
   * colors.
   */
  function dispatchChangedEvent() {
    const colorArray = [];
    document
      .querySelectorAll('#altcolor_form input[type="color"]')
      .forEach((colorInput) => {
        // Convert the nested form name into the property name.
        let colorName = colorInput.name;
        const index = colorName.lastIndexOf('[');
        colorName = colorName.slice(index + 1, -1);
        colorArray[colorName] = colorInput.value;
      });
    const eventChanged = new CustomEvent('altcolor-changed', {
      detail: { colors: colorArray },
    });
    document.querySelector('#altcolor_form').dispatchEvent(eventChanged);
  }

  /**
   * Updates all color elements once a preset color scheme is chosen.
   * @param event - The change event.
   */
  function setColorScheme(event) {
    const { target } = event;
    if (!target.value) return;
    const selectedColorScheme = settings.altcolor.schemes[target.value].colors;
    if (selectedColorScheme) {
      Object.entries(selectedColorScheme).forEach((scheme) => {
        const [key, color] = scheme;
        document
          .querySelectorAll(`input[name="altcolor[colors][${key}]"]`)
          .forEach((input) => {
            if (input.value !== color) {
              input.value = color;
            }
          });
      });
      dispatchChangedEvent();
    }
  }

  /**
   * Initializes the color scheme selection element (selectElement).
   * @param selectElement - The element to initialize.
   */
  function initColorSchemeSelect(selectElement) {
    // Add an event listener to the style selection input element, to allow the
    // individual color input elements to change if the style selection
    // element's value changes.
    selectElement.addEventListener('change', setColorScheme);

    // Iterate over all color schemes and see if the current color values match
    // with a known scheme. If so, set the selectElement's value to the scheme.
    Object.entries(settings.altcolor.schemes).forEach((option) => {
      const [scheme, values] = option;
      const { label, colors } = values;
      let allColorsMatch = true;
      Object.entries(colors).forEach((col) => {
        const [colorName, colorHex] = col;
        const field = document.querySelector(
          `input[type="color"][name="altcolor[colors][${colorName}]"]`,
        );
        if (field.value !== colorHex) {
          allColorsMatch = false;
        }
      });

      if (allColorsMatch) {
        selectElement.value = scheme;
      }
    });
  }

  /**
   * Changes the selectElement to have an empty value.
   */
  function unsetSchemeSelect() {
    const selectElement = document.querySelector('#altcolor_form select');
    if (selectElement.value !== '') {
      selectElement.value = '';
    }
  }

  /**
   * Initializes the Color Alt administration interface.
   *
   * @type {Drupal~behavior}
   *
   * @prop {Drupal~behaviorAttach} attach
   *   Attaches the behavior for Color Alt.
   */
  Drupal.behaviors.colorAlt = {
    attach: () => {
      const colorSchemeSelect = once('altcolor', '#altcolor_form select');
      colorSchemeSelect.forEach((selectElement) => {
        initColorSchemeSelect(selectElement);
      });
      const colorTextInputs = once(
        'altcolor',
        '#altcolor_form input[type="color"]',
      );
      colorTextInputs.forEach((colorInput) => {
        colorInput.addEventListener('input', unsetSchemeSelect);
        colorInput.addEventListener('input', dispatchChangedEvent);
      });

      // Allow the preview to update with the values loaded from configuration.
      dispatchChangedEvent();
    },
  };

  /**
   * Helper to allow JavaScript to update the preview.
   *
   * Add altcolor to the Drupal object so that we can request an update from
   * JavaScript outside of this file.
   */
  Drupal.altcolor = {
    update() {
      dispatchChangedEvent();
    },
  };
})(Drupal, drupalSettings, once);
