<?php

namespace Drupal\altcolor\Form;

use Drupal\altcolor\ColorFetcher;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\TypedConfigManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Routing\UrlGeneratorInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class for building a ColorForm.
 */
class ColorForm extends ConfigFormBase {

  /**
   * The color fetcher.
   *
   * @var \Drupal\altcolor\ColorFetcher
   */
  private ColorFetcher $colorFetcher;

  /**
   * The URL generator.
   *
   * @var \Drupal\Core\Routing\UrlGeneratorInterface
   */
  private UrlGeneratorInterface $urlGenerator;

  /**
   * An array of configuration names that should be editable.
   *
   * @var array
   */
  protected $editableConfig = [];

  /**
   * Constructs a \Drupal\system\ConfigFormBase object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Config\TypedConfigManagerInterface $typedConfigManager
   *   The typed configuration manager.
   * @param \Drupal\altcolor\ColorFetcher $color_fetcher
   *   The color fetcher.
   * @param \Drupal\Core\Routing\UrlGeneratorInterface $url_generator
   *   The URL generator.
   */
  public function __construct(ConfigFactoryInterface $config_factory, TypedConfigManagerInterface $typedConfigManager, ColorFetcher $color_fetcher, UrlGeneratorInterface $url_generator) {
    parent::__construct($config_factory, $typedConfigManager);
    $this->colorFetcher = $color_fetcher;
    $this->urlGenerator = $url_generator;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('config.typed'),
      $container->get('altcolor.fetcher'),
      $container->get('url_generator')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return $this->editableConfig;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'altcolor_settings';
  }

  /**
   * {@inheritDoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $theme = '') {
    $config_key = $theme . '.settings';
    $this->editableConfig = [$config_key];

    // Define $colors and $schemes.
    ['colors' => $colors, 'schemes' => $schemes] = $this->colorFetcher->fetch($theme);

    $form['altcolor'] = [
      '#type' => 'details',
      '#title' => t('Color scheme'),
      '#open' => TRUE,
      '#attributes' => [
        'id' => 'altcolor_form',
      ],
      '#attached' => [
        'library' => [
          'altcolor/altcolor.admin',
        ],
        'drupalSettings' => [
          'altcolor' => [
            'schemes' => $schemes,
          ],
        ],
      ],
    ];

    $form['altcolor']['colors'] = [
      '#type' => 'container',
    ];

    $form['altcolor']['colors']['scheme'] = [
      '#type' => 'select',
      '#title' => $this->t('Color scheme'),
      '#options' => array_combine(array_keys($schemes), array_column($schemes, 'label')),
      '#empty_option' => $this->t('Custom'),
      '#empty_value' => NULL,
    ];

    $form['altcolor']['colors']['colors'] = [
      '#type' => 'container',
      '#title' => $this->t('Colors'),
      '#tree' => TRUE,
      '#parents' => ['altcolor', 'colors'],
    ];
    foreach ($colors as $variable => $label) {
      // If the color is not yet configured, try to fetch it from the first
      // color scheme.
      $fallback_color = array_values($schemes)[0]['colors'][$variable] ?? NULL;
      $form['altcolor']['colors']['colors'][$variable] = [
        '#type' => 'color',
        '#title' => $label,
        '#default_value' => $this->altcolorPaletteColorValue($variable, $fallback_color),
      ];
    }

    $form['altcolor']['preview'] = [
      '#type' => 'container',
    ];

    $form['altcolor']['preview']['preview'] = [
      '#type' => 'html_tag',
      '#tag' => 'iframe',
      '#attributes' => [
        // The path contains the altcolor_preview_theme query parameter,
        // allowing this module's theme negotiator to force a theme for the
        // preview within the iFrame. This renders the front page in the same
        // theme this settings page is about.
        'src' => $this->urlGenerator->generateFromRoute(
          '<front>',
          [],
          ['absolute' => TRUE, 'query' => ['altcolor_preview_theme' => $theme]]
        ),
        'allow' => 'fullscreen',
        'name' => 'altcolor-preview-frame',
        'style' => 'width:100%; height: 100%; min-height: 650px; zoom: 0.8;',
      ],
      '#prefix' => '<strong>This is the real website. Any changes made here will have an immediate effect.</strong>',
      '#attached' => [
        'library' => [
          'altcolor/altcolor.preview',
        ],
      ],
    ];

    return $form;
  }

  /**
   * Determines the value for a color field.
   *
   * @return string
   *   The HEX color value to set for the field.
   */
  public function altcolorPaletteColorValue($key, $fallback_value) {
    $color_settings = $this->config($this->editableConfig[0])->get('third_party_settings.altcolor.colors');
    if (!empty($color_settings[$key])) {
      return $color_settings[$key];
    }
    return $fallback_value;
  }

}
